@@ -13,6 +13,7 @@
 # limitations under the License.
 
 """Code for constructing the model."""
+
 from typing import Any, Mapping, Optional, Union
 
 from absl import logging
@@ -22,41 +23,49 @@ from alphafold.model import modules
 from alphafold.model import modules_multimer
 import haiku as hk
 import jax
+from jax import tree
 import ml_collections
 import numpy as np
 import tensorflow.compat.v1 as tf
-import tree
 
 
 def get_confidence_metrics(
-    prediction_result: Mapping[str, Any],
-    multimer_mode: bool) -> Mapping[str, Any]:
+    prediction_result: Mapping[str, Any], multimer_mode: bool
+) -> Mapping[str, Any]:
   """Post processes prediction_result to get confidence metrics."""
   confidence_metrics = {}
   confidence_metrics['plddt'] = confidence.compute_plddt(
-      prediction_result['predicted_lddt']['logits'])
+      prediction_result['predicted_lddt']['logits']
+  )
   if 'predicted_aligned_error' in prediction_result:
-    confidence_metrics.update(confidence.compute_predicted_aligned_error(
-        logits=prediction_result['predicted_aligned_error']['logits'],
-        breaks=prediction_result['predicted_aligned_error']['breaks']))
+    confidence_metrics.update(
+        confidence.compute_predicted_aligned_error(
+            logits=prediction_result['predicted_aligned_error']['logits'],
+            breaks=prediction_result['predicted_aligned_error']['breaks'],
+        )
+    )
     confidence_metrics['ptm'] = confidence.predicted_tm_score(
         logits=prediction_result['predicted_aligned_error']['logits'],
         breaks=prediction_result['predicted_aligned_error']['breaks'],
-        asym_id=None)
+        asym_id=None,
+    )
     if multimer_mode:
       # Compute the ipTM only for the multimer model.
       confidence_metrics['iptm'] = confidence.predicted_tm_score(
           logits=prediction_result['predicted_aligned_error']['logits'],
           breaks=prediction_result['predicted_aligned_error']['breaks'],
           asym_id=prediction_result['predicted_aligned_error']['asym_id'],
-          interface=True)
+          interface=True,
+      )
       confidence_metrics['ranking_confidence'] = (
-          0.8 * confidence_metrics['iptm'] + 0.2 * confidence_metrics['ptm'])
+          0.8 * confidence_metrics['iptm'] + 0.2 * confidence_metrics['ptm']
+      )
 
   if not multimer_mode:
     # Monomer models use mean pLDDT for model ranking.
     confidence_metrics['ranking_confidence'] = np.mean(
-        confidence_metrics['plddt'])
+        confidence_metrics['plddt']
+    )
 
   return confidence_metrics
 
@@ -64,27 +73,31 @@ def get_confidence_metrics(
 class RunModel:
   """Container for JAX model."""
 
-  def __init__(self,
-               config: ml_collections.ConfigDict,
-               params: Optional[Mapping[str, Mapping[str, jax.Array]]] = None):
+  def __init__(
+      self,
+      config: ml_collections.ConfigDict,
+      params: Optional[Mapping[str, Mapping[str, jax.Array]]] = None,
+  ):
     self.config = config
     self.params = params
     self.multimer_mode = config.model.global_config.multimer_mode
 
     if self.multimer_mode:
+
       def _forward_fn(batch):
         model = modules_multimer.AlphaFold(self.config.model)
-        return model(
-            batch,
-            is_training=False)
+        return model(batch, is_training=False)
+
     else:
+
       def _forward_fn(batch):
         model = modules.AlphaFold(self.config.model)
         return model(
             batch,
             is_training=False,
             compute_loss=False,
-            ensemble_representations=True)
+            ensemble_representations=True,
+        )
 
     self.apply = jax.jit(hk.transform(_forward_fn).apply)
     self.init = jax.jit(hk.transform(_forward_fn).init)
@@ -104,14 +117,14 @@ class RunModel:
     if not self.params:
       # Init params randomly.
       rng = jax.random.PRNGKey(random_seed)
-      self.params = hk.data_structures.to_mutable_dict(
-          self.init(rng, feat))
+      self.params = hk.data_structures.to_mutable_dict(self.init(rng, feat))
       logging.warning('Initialized parameters randomly')
 
   def process_features(
       self,
       raw_features: Union[tf.train.Example, features.FeatureDict],
-      random_seed: int) -> features.FeatureDict:
+      random_seed: int,
+  ) -> features.FeatureDict:
     """Processes features to prepare for feeding them into the model.
 
     Args:
@@ -129,27 +142,28 @@ class RunModel:
     # Single-chain mode.
     if isinstance(raw_features, dict):
       return features.np_example_to_features(
-          np_example=raw_features,
-          config=self.config,
-          random_seed=random_seed)
+          np_example=raw_features, config=self.config, random_seed=random_seed
+      )
     else:
       return features.tf_example_to_features(
-          tf_example=raw_features,
-          config=self.config,
-          random_seed=random_seed)
+          tf_example=raw_features, config=self.config, random_seed=random_seed
+      )
 
   def eval_shape(self, feat: features.FeatureDict) -> jax.ShapeDtypeStruct:
     self.init_params(feat)
-    logging.info('Running eval_shape with shape(feat) = %s',
-                 tree.map_structure(lambda x: x.shape, feat))
+    logging.info(
+        'Running eval_shape with shape(feat) = %s',
+        tree.map(lambda x: x.shape, feat),
+    )
     shape = jax.eval_shape(self.apply, self.params, jax.random.PRNGKey(0), feat)
     logging.info('Output shape was %s', shape)
     return shape
 
-  def predict(self,
-              feat: features.FeatureDict,
-              random_seed: int,
-              ) -> Mapping[str, Any]:
+  def predict(
+      self,
+      feat: features.FeatureDict,
+      random_seed: int,
+  ) -> Mapping[str, Any]:
     """Makes a prediction by inferencing the model on the provided features.
 
     Args:
@@ -162,18 +176,18 @@ class RunModel:
       A dictionary of model outputs.
     """
     self.init_params(feat)
-    logging.info('Running predict with shape(feat) = %s',
-                 tree.map_structure(lambda x: x.shape, feat))
+    logging.info(
+        'Running predict with shape(feat) = %s',
+        tree.map(lambda x: x.shape, feat),
+    )
     result = self.apply(self.params, jax.random.PRNGKey(random_seed), feat)
 
     # This block is to ensure benchmark timings are accurate. Some blocking is
     # already happening when computing get_confidence_metrics, and this ensures
     # all outputs are blocked on.
-    jax.tree_map(lambda x: x.block_until_ready(), result)
+    tree.map(lambda x: x.block_until_ready(), result)
     result.update(
-        get_confidence_metrics(result, multimer_mode=self.multimer_mode))
-    result.update({"plddt": confidence.compute_plddt(
-      result['predicted_lddt']['logits'])})
-    logging.info('Output shape was %s',
-                 tree.map_structure(lambda x: x.shape, result))
+        get_confidence_metrics(result, multimer_mode=self.multimer_mode)
+    )
+    logging.info('Output shape was %s', tree.map(lambda x: x.shape, result))
     return result
