@@ -55,95 +55,213 @@ class ModelsToRelax(enum.Enum):
   BEST = 1
   NONE = 2
 
+
 flags.DEFINE_list(
-    'fasta_paths', None, 'Paths to FASTA files, each containing a prediction '
+    'fasta_paths',
+    None,
+    'Paths to FASTA files, each containing a prediction '
     'target that will be folded one after another. If a FASTA file contains '
     'multiple sequences, then it will be folded as a multimer. Paths should be '
     'separated by commas. All FASTA paths must have a unique basename as the '
-    'basename is used to name the output directories for each prediction.')
+    'basename is used to name the output directories for each prediction.',
+)
 
 flags.DEFINE_string('data_dir', None, 'Path to directory of supporting data.')
-flags.DEFINE_string('output_dir', None, 'Path to a directory that will '
-                    'store the results.')
-flags.DEFINE_string('jackhmmer_binary_path', shutil.which('jackhmmer'),
-                    'Path to the JackHMMER executable.')
-flags.DEFINE_string('hhblits_binary_path', shutil.which('hhblits'),
-                    'Path to the HHblits executable.')
-flags.DEFINE_string('hhsearch_binary_path', shutil.which('hhsearch'),
-                    'Path to the HHsearch executable.')
-flags.DEFINE_string('hmmsearch_binary_path', shutil.which('hmmsearch'),
-                    'Path to the hmmsearch executable.')
-flags.DEFINE_string('hmmbuild_binary_path', shutil.which('hmmbuild'),
-                    'Path to the hmmbuild executable.')
-flags.DEFINE_string('kalign_binary_path', shutil.which('kalign'),
-                    'Path to the Kalign executable.')
-flags.DEFINE_string('uniref90_database_path', None, 'Path to the Uniref90 '
-                    'database for use by JackHMMER.')
-flags.DEFINE_string('mgnify_database_path', None, 'Path to the MGnify '
-                    'database for use by JackHMMER.')
-flags.DEFINE_string('bfd_database_path', None, 'Path to the BFD '
-                    'database for use by HHblits.')
-flags.DEFINE_string('small_bfd_database_path', None, 'Path to the small '
-                    'version of BFD used with the "reduced_dbs" preset.')
-flags.DEFINE_string('uniref30_database_path', None, 'Path to the UniRef30 '
-                    'database for use by HHblits.')
-flags.DEFINE_string('uniprot_database_path', None, 'Path to the Uniprot '
-                    'database for use by JackHMMer.')
-flags.DEFINE_string('pdb70_database_path', None, 'Path to the PDB70 '
-                    'database for use by HHsearch.')
-flags.DEFINE_string('pdb_seqres_database_path', None, 'Path to the PDB '
-                    'seqres database for use by hmmsearch.')
-flags.DEFINE_string('template_mmcif_dir', None, 'Path to a directory with '
-                    'template mmCIF structures, each named <pdb_id>.cif')
-flags.DEFINE_string('max_template_date', None, 'Maximum template release date '
-                    'to consider. Important if folding historical test sets.')
-flags.DEFINE_string('obsolete_pdbs_path', None, 'Path to file containing a '
-                    'mapping from obsolete PDB IDs to the PDB IDs of their '
-                    'replacements.')
-flags.DEFINE_enum('db_preset', 'full_dbs',
-                  ['full_dbs', 'reduced_dbs'],
-                  'Choose preset MSA database configuration - '
-                  'smaller genetic database config (reduced_dbs) or '
-                  'full genetic database config  (full_dbs)')
-flags.DEFINE_enum('model_preset', 'monomer',
-                  ['monomer', 'monomer_casp14', 'monomer_ptm', 'multimer'],
-                  'Choose preset model configuration - the monomer model, '
-                  'the monomer model with extra ensembling, monomer model with '
-                  'pTM head, or multimer model')
-flags.DEFINE_boolean('benchmark', False, 'Run multiple JAX model evaluations '
-                     'to obtain a timing that excludes the compilation time, '
-                     'which should be more indicative of the time required for '
-                     'inferencing many proteins.')
-flags.DEFINE_integer('random_seed', None, 'The random seed for the data '
-                     'pipeline. By default, this is randomly generated. Note '
-                     'that even if this is set, Alphafold may still not be '
-                     'deterministic, because processes like GPU inference are '
-                     'nondeterministic.')
-flags.DEFINE_integer('num_multimer_predictions_per_model', 5, 'How many '
-                     'predictions (each with a different random seed) will be '
-                     'generated per model. E.g. if this is 2 and there are 5 '
-                     'models then there will be 10 predictions per input. '
-                     'Note: this FLAG only applies if model_preset=multimer')
-flags.DEFINE_boolean('use_precomputed_msas', False, 'Whether to read MSAs that '
-                     'have been written to disk instead of running the MSA '
-                     'tools. The MSA files are looked up in the output '
-                     'directory, so it must stay the same between multiple '
-                     'runs that are to reuse the MSAs. WARNING: This will not '
-                     'check if the sequence, database or configuration have '
-                     'changed.')
-flags.DEFINE_enum_class('models_to_relax', ModelsToRelax.BEST, ModelsToRelax,
-                        'The models to run the final relaxation step on. '
-                        'If `all`, all models are relaxed, which may be time '
-                        'consuming. If `best`, only the most confident model '
-                        'is relaxed. If `none`, relaxation is not run. Turning '
-                        'off relaxation might result in predictions with '
-                        'distracting stereochemical violations but might help '
-                        'in case you are having issues with the relaxation '
-                        'stage.')
-flags.DEFINE_boolean('use_gpu_relax', None, 'Whether to relax on GPU. '
-                     'Relax on GPU can be much faster than CPU, so it is '
-                     'recommended to enable if possible. GPUs must be available'
-                     ' if this setting is enabled.')
+flags.DEFINE_string(
+    'output_dir', None, 'Path to a directory that will store the results.'
+)
+flags.DEFINE_string(
+    'jackhmmer_binary_path',
+    shutil.which('jackhmmer'),
+    'Path to the JackHMMER executable.',
+)
+flags.DEFINE_string(
+    'hhblits_binary_path',
+    shutil.which('hhblits'),
+    'Path to the HHblits executable.',
+)
+flags.DEFINE_string(
+    'hhsearch_binary_path',
+    shutil.which('hhsearch'),
+    'Path to the HHsearch executable.',
+)
+flags.DEFINE_string(
+    'hmmsearch_binary_path',
+    shutil.which('hmmsearch'),
+    'Path to the hmmsearch executable.',
+)
+flags.DEFINE_string(
+    'hmmbuild_binary_path',
+    shutil.which('hmmbuild'),
+    'Path to the hmmbuild executable.',
+)
+flags.DEFINE_string(
+    'kalign_binary_path',
+    shutil.which('kalign'),
+    'Path to the Kalign executable.',
+)
+flags.DEFINE_string(
+    'uniref90_database_path',
+    None,
+    'Path to the Uniref90 database for use by JackHMMER.',
+)
+flags.DEFINE_string(
+    'mgnify_database_path',
+    None,
+    'Path to the MGnify database for use by JackHMMER.',
+)
+flags.DEFINE_string(
+    'bfd_database_path', None, 'Path to the BFD database for use by HHblits.'
+)
+flags.DEFINE_string(
+    'small_bfd_database_path',
+    None,
+    'Path to the small version of BFD used with the "reduced_dbs" preset.',
+)
+flags.DEFINE_string(
+    'uniref30_database_path',
+    None,
+    'Path to the UniRef30 database for use by HHblits.',
+)
+flags.DEFINE_string(
+    'uniprot_database_path',
+    None,
+    'Path to the Uniprot database for use by JackHMMer.',
+)
+flags.DEFINE_string(
+    'pdb70_database_path',
+    None,
+    'Path to the PDB70 database for use by HHsearch.',
+)
+flags.DEFINE_string(
+    'pdb_seqres_database_path',
+    None,
+    'Full filepath to the '
+    'PDB seqres database file (not just the directory) for use '
+    'by hmmsearch.',
+)
+flags.DEFINE_string(
+    'template_mmcif_dir',
+    None,
+    'Path to a directory with '
+    'template mmCIF structures, each named <pdb_id>.cif',
+)
+flags.DEFINE_string(
+    'max_template_date',
+    None,
+    'Maximum template release date '
+    'to consider. Important if folding historical test sets.',
+)
+flags.DEFINE_string(
+    'obsolete_pdbs_path',
+    None,
+    'Path to file containing a '
+    'mapping from obsolete PDB IDs to the PDB IDs of their '
+    'replacements.',
+)
+flags.DEFINE_enum(
+    'db_preset',
+    'full_dbs',
+    ['full_dbs', 'reduced_dbs'],
+    'Choose preset MSA database configuration - '
+    'smaller genetic database config (reduced_dbs) or '
+    'full genetic database config  (full_dbs)',
+)
+flags.DEFINE_enum(
+    'model_preset',
+    'monomer',
+    ['monomer', 'monomer_casp14', 'monomer_ptm', 'multimer'],
+    'Choose preset model configuration - the monomer model, '
+    'the monomer model with extra ensembling, monomer model with '
+    'pTM head, or multimer model',
+)
+flags.DEFINE_boolean(
+    'benchmark',
+    False,
+    'Run multiple JAX model evaluations '
+    'to obtain a timing that excludes the compilation time, '
+    'which should be more indicative of the time required for '
+    'inferencing many proteins.',
+)
+flags.DEFINE_integer(
+    'random_seed',
+    None,
+    'The random seed for the data '
+    'pipeline. By default, this is randomly generated. Note '
+    'that even if this is set, Alphafold may still not be '
+    'deterministic, because processes like GPU inference are '
+    'nondeterministic.',
+)
+flags.DEFINE_integer(
+    'num_multimer_predictions_per_model',
+    5,
+    'How many '
+    'predictions (each with a different random seed) will be '
+    'generated per model. E.g. if this is 2 and there are 5 '
+    'models then there will be 10 predictions per input. '
+    'Note: this FLAG only applies if model_preset=multimer',
+)
+flags.DEFINE_boolean(
+    'use_precomputed_msas',
+    False,
+    'Whether to read MSAs that '
+    'have been written to disk instead of running the MSA '
+    'tools. The MSA files are looked up in the output '
+    'directory, so it must stay the same between multiple '
+    'runs that are to reuse the MSAs. WARNING: This will not '
+    'check if the sequence, database or configuration have '
+    'changed.',
+)
+flags.DEFINE_enum_class(
+    'models_to_relax',
+    ModelsToRelax.BEST,
+    ModelsToRelax,
+    'The models to run the final relaxation step on. '
+    'If `all`, all models are relaxed, which may be time '
+    'consuming. If `best`, only the most confident model '
+    'is relaxed. If `none`, relaxation is not run. Turning '
+    'off relaxation might result in predictions with '
+    'distracting stereochemical violations but might help '
+    'in case you are having issues with the relaxation '
+    'stage.',
+)
+flags.DEFINE_boolean(
+    'use_gpu_relax',
+    None,
+    'Whether to relax on GPU. '
+    'Relax on GPU can be much faster than CPU, so it is '
+    'recommended to enable if possible. GPUs must be available'
+    ' if this setting is enabled.',
+)
+flags.DEFINE_integer(
+    'jackhmmer_n_cpu',
+    # Unfortunately, os.process_cpu_count() is only available in Python 3.13+.
+    min(len(os.sched_getaffinity(0)), 8),
+    'Number of CPUs to use for Jackhmmer. Defaults to min(cpu_count, 8). Going'
+    ' above 8 CPUs provides very little additional speedup.',
+    lower_bound=0,
+)
+flags.DEFINE_integer(
+    'hmmsearch_n_cpu',
+    # Unfortunately, os.process_cpu_count() is only available in Python 3.13+.
+    min(len(os.sched_getaffinity(0)), 8),
+    'Number of CPUs to use for HMMsearch. Defaults to min(cpu_count, 8). Going'
+    ' above 8 CPUs provides very little additional speedup.',
+    lower_bound=0,
+)
+flags.DEFINE_integer(
+    'hhsearch_n_cpu',
+    # Unfortunately, os.process_cpu_count() is only available in Python 3.13+.
+    min(len(os.sched_getaffinity(0)), 8),
+    'Number of CPUs to use for HHsearch. Defaults to min(cpu_count, 8). Going'
+    ' above 8 CPUs provides very little additional speedup.',
+    lower_bound=0,
+)
+
+flags.DEFINE_boolean(
+    'features_only', False,
+    'If true, stop after writing features.pkl and skip all model/relax steps.'
+)
 
 FLAGS = flags.FLAGS
 
@@ -155,13 +273,13 @@ RELAX_EXCLUDE_RESIDUES = []
 RELAX_MAX_OUTER_ITERATIONS = 3
 
 
-def _check_flag(flag_name: str,
-                other_flag_name: str,
-                should_be_set: bool):
+def _check_flag(flag_name: str, other_flag_name: str, should_be_set: bool):
   if should_be_set != bool(FLAGS[flag_name].value):
     verb = 'be' if should_be_set else 'not be'
-    raise ValueError(f'{flag_name} must {verb} set when running with '
-                     f'"--{other_flag_name}={FLAGS[other_flag_name].value}".')
+    raise ValueError(
+        f'{flag_name} must {verb} set when running with '
+        f'"--{other_flag_name}={FLAGS[other_flag_name].value}".'
+    )
 
 
 def _jnp_to_np(output: Dict[str, Any]) -> Dict[str, Any]:
@@ -194,7 +312,7 @@ def _save_mmcif_file(
     file_id: str,
     model_type: str,
 ) -> None:
-  """Crate mmCIF string and save to a file.
+  """Create mmCIF string and save to a file.
 
   Args:
     prot: Protein object.
@@ -256,15 +374,17 @@ def predict_structure(
   # Get features.
   t_0 = time.time()
   feature_dict = data_pipeline.process(
-      input_fasta_path=fasta_path,
-      msa_output_dir=msa_output_dir)
+      input_fasta_path=fasta_path, msa_output_dir=msa_output_dir
+  )
   timings['features'] = time.time() - t_0
 
   # Write out features as a pickled dictionary.
   features_output_path = os.path.join(output_dir, 'features.pkl')
   with open(features_output_path, 'wb') as f:
     pickle.dump(feature_dict, f, protocol=4)
-
+  if FLAGS.features_only:
+    logging.info('Features only, exiting after writing features.pkl')
+    return
   unrelaxed_pdbs = {}
   unrelaxed_proteins = {}
   relaxed_pdbs = {}
@@ -274,32 +394,44 @@ def predict_structure(
   # Run the models.
   num_models = len(model_runners)
   for model_index, (model_name, model_runner) in enumerate(
-      model_runners.items()):
+      model_runners.items()
+  ):
     logging.info('Running model %s on %s', model_name, fasta_name)
     t_0 = time.time()
     model_random_seed = model_index + random_seed * num_models
     processed_feature_dict = model_runner.process_features(
-        feature_dict, random_seed=model_random_seed)
+        feature_dict, random_seed=model_random_seed
+    )
     timings[f'process_features_{model_name}'] = time.time() - t_0
 
     t_0 = time.time()
-    prediction_result = model_runner.predict(processed_feature_dict,
-                                             random_seed=model_random_seed)
+    prediction_result = model_runner.predict(
+        processed_feature_dict, random_seed=model_random_seed
+    )
     t_diff = time.time() - t_0
     timings[f'predict_and_compile_{model_name}'] = t_diff
     logging.info(
-        'Total JAX model %s on %s predict time (includes compilation time, see --benchmark): %.1fs',
-        model_name, fasta_name, t_diff)
+        'Total JAX model %s on %s predict time (includes compilation time, see'
+        ' --benchmark): %.1fs',
+        model_name,
+        fasta_name,
+        t_diff,
+    )
 
     if benchmark:
       t_0 = time.time()
-      model_runner.predict(processed_feature_dict,
-                           random_seed=model_random_seed)
+      model_runner.predict(
+          processed_feature_dict, random_seed=model_random_seed
+      )
       t_diff = time.time() - t_0
       timings[f'predict_benchmark_{model_name}'] = t_diff
       logging.info(
-          'Total JAX model %s on %s predict time (excludes compilation time): %.1fs',
-          model_name, fasta_name, t_diff)
+          'Total JAX model %s on %s predict time (excludes compilation time):'
+          ' %.1fs',
+          model_name,
+          fasta_name,
+          t_diff,
+      )
 
     plddt = prediction_result['plddt']
     _save_confidence_json_file(plddt, output_dir, model_name)
@@ -324,12 +456,14 @@ def predict_structure(
     # Add the predicted LDDT in the b-factor column.
     # Note that higher predicted LDDT value means higher model confidence.
     plddt_b_factors = np.repeat(
-        plddt[:, None], residue_constants.atom_type_num, axis=-1)
+        plddt[:, None], residue_constants.atom_type_num, axis=-1
+    )
     unrelaxed_protein = protein.from_prediction(
         features=processed_feature_dict,
         result=prediction_result,
         b_factors=plddt_b_factors,
-        remove_leading_feature_dimension=not model_runner.multimer_mode)
+        remove_leading_feature_dimension=not model_runner.multimer_mode,
+    )
 
     unrelaxed_proteins[model_name] = unrelaxed_protein
     unrelaxed_pdbs[model_name] = protein.to_pdb(unrelaxed_protein)
@@ -347,8 +481,11 @@ def predict_structure(
 
   # Rank by model confidence.
   ranked_order = [
-      model_name for model_name, confidence in
-      sorted(ranking_confidences.items(), key=lambda x: x[1], reverse=True)]
+      model_name
+      for model_name, confidence in sorted(
+          ranking_confidences.items(), key=lambda x: x[1], reverse=True
+      )
+  ]
 
   # Relax predictions.
   if models_to_relax == ModelsToRelax.BEST:
@@ -361,18 +498,18 @@ def predict_structure(
   for model_name in to_relax:
     t_0 = time.time()
     relaxed_pdb_str, _, violations = amber_relaxer.process(
-        prot=unrelaxed_proteins[model_name])
+        prot=unrelaxed_proteins[model_name]
+    )
     relax_metrics[model_name] = {
         'remaining_violations': violations,
-        'remaining_violations_count': sum(violations)
+        'remaining_violations_count': sum(violations),
     }
     timings[f'relax_{model_name}'] = time.time() - t_0
 
     relaxed_pdbs[model_name] = relaxed_pdb_str
 
     # Save the relaxed PDB.
-    relaxed_output_path = os.path.join(
-        output_dir, f'relaxed_{model_name}.pdb')
+    relaxed_output_path = os.path.join(output_dir, f'relaxed_{model_name}.pdb')
     with open(relaxed_output_path, 'w') as f:
       f.write(relaxed_pdb_str)
 
@@ -410,8 +547,11 @@ def predict_structure(
   ranking_output_path = os.path.join(output_dir, 'ranking_debug.json')
   with open(ranking_output_path, 'w') as f:
     label = 'iptm+ptm' if 'iptm' in prediction_result else 'plddts'
-    f.write(json.dumps(
-        {label: ranking_confidences, 'order': ranked_order}, indent=4))
+    f.write(
+        json.dumps(
+            {label: ranking_confidences, 'order': ranked_order}, indent=4
+        )
+    )
 
   logging.info('Final timings for %s: %s', fasta_name, timings)
 
@@ -429,32 +569,43 @@ def main(argv):
     raise app.UsageError('Too many command-line arguments.')
 
   for tool_name in (
-      'jackhmmer', 'hhblits', 'hhsearch', 'hmmsearch', 'hmmbuild', 'kalign'):
+      'jackhmmer',
+      'hhblits',
+      'hhsearch',
+      'hmmsearch',
+      'hmmbuild',
+      'kalign',
+  ):
     if not FLAGS[f'{tool_name}_binary_path'].value:
-      raise ValueError(f'Could not find path to the "{tool_name}" binary. Make '
-                       'sure it is installed on your system.')
+      raise ValueError(
+          f'Could not find path to the "{tool_name}" binary. Make '
+          'sure it is installed on your system.'
+      )
 
   use_small_bfd = FLAGS.db_preset == 'reduced_dbs'
-  _check_flag('small_bfd_database_path', 'db_preset',
-              should_be_set=use_small_bfd)
-  _check_flag('bfd_database_path', 'db_preset',
-              should_be_set=not use_small_bfd)
-  _check_flag('uniref30_database_path', 'db_preset',
-              should_be_set=not use_small_bfd)
+  _check_flag(
+      'small_bfd_database_path', 'db_preset', should_be_set=use_small_bfd
+  )
+  _check_flag('bfd_database_path', 'db_preset', should_be_set=not use_small_bfd)
+  _check_flag(
+      'uniref30_database_path', 'db_preset', should_be_set=not use_small_bfd
+  )
 
   run_multimer_system = 'multimer' in FLAGS.model_preset
   model_type = 'Multimer' if run_multimer_system else 'Monomer'
-  _check_flag('pdb70_database_path', 'model_preset',
-              should_be_set=not run_multimer_system)
-  _check_flag('pdb_seqres_database_path', 'model_preset',
-              should_be_set=run_multimer_system)
-  _check_flag('uniprot_database_path', 'model_preset',
-              should_be_set=run_multimer_system)
-
-  if FLAGS.model_preset == 'monomer_casp14':
-    num_ensemble = 8
-  else:
-    num_ensemble = 1
+  _check_flag(
+      'pdb70_database_path',
+      'model_preset',
+      should_be_set=not run_multimer_system,
+  )
+  _check_flag(
+      'pdb_seqres_database_path',
+      'model_preset',
+      should_be_set=run_multimer_system,
+  )
+  _check_flag(
+      'uniprot_database_path', 'model_preset', should_be_set=run_multimer_system
+  )
 
   # Check for duplicate FASTA file names.
   fasta_names = [pathlib.Path(p).stem for p in FLAGS.fasta_paths]
@@ -465,25 +616,31 @@ def main(argv):
     template_searcher = hmmsearch.Hmmsearch(
         binary_path=FLAGS.hmmsearch_binary_path,
         hmmbuild_binary_path=FLAGS.hmmbuild_binary_path,
-        database_path=FLAGS.pdb_seqres_database_path)
+        database_path=FLAGS.pdb_seqres_database_path,
+        cpu=FLAGS.hmmsearch_n_cpu,
+    )
     template_featurizer = templates.HmmsearchHitFeaturizer(
         mmcif_dir=FLAGS.template_mmcif_dir,
         max_template_date=FLAGS.max_template_date,
         max_hits=MAX_TEMPLATE_HITS,
         kalign_binary_path=FLAGS.kalign_binary_path,
         release_dates_path=None,
-        obsolete_pdbs_path=FLAGS.obsolete_pdbs_path)
+        obsolete_pdbs_path=FLAGS.obsolete_pdbs_path,
+    )
   else:
     template_searcher = hhsearch.HHSearch(
         binary_path=FLAGS.hhsearch_binary_path,
-        databases=[FLAGS.pdb70_database_path])
+        databases=[FLAGS.pdb70_database_path],
+        cpu=FLAGS.hhsearch_n_cpu,
+    )
     template_featurizer = templates.HhsearchHitFeaturizer(
         mmcif_dir=FLAGS.template_mmcif_dir,
         max_template_date=FLAGS.max_template_date,
         max_hits=MAX_TEMPLATE_HITS,
         kalign_binary_path=FLAGS.kalign_binary_path,
         release_dates_path=None,
-        obsolete_pdbs_path=FLAGS.obsolete_pdbs_path)
+        obsolete_pdbs_path=FLAGS.obsolete_pdbs_path,
+    )
 
   monomer_data_pipeline = pipeline.DataPipeline(
       jackhmmer_binary_path=FLAGS.jackhmmer_binary_path,
@@ -496,7 +653,9 @@ def main(argv):
       template_searcher=template_searcher,
       template_featurizer=template_featurizer,
       use_small_bfd=use_small_bfd,
-      use_precomputed_msas=FLAGS.use_precomputed_msas)
+      use_precomputed_msas=FLAGS.use_precomputed_msas,
+      msa_tools_n_cpu=FLAGS.jackhmmer_n_cpu,
+  )
 
   if run_multimer_system:
     num_predictions_per_model = FLAGS.num_multimer_predictions_per_model
@@ -504,7 +663,9 @@ def main(argv):
         monomer_data_pipeline=monomer_data_pipeline,
         jackhmmer_binary_path=FLAGS.jackhmmer_binary_path,
         uniprot_database_path=FLAGS.uniprot_database_path,
-        use_precomputed_msas=FLAGS.use_precomputed_msas)
+        use_precomputed_msas=FLAGS.use_precomputed_msas,
+        jackhmmer_n_cpu=FLAGS.jackhmmer_n_cpu,
+    )
   else:
     num_predictions_per_model = 1
     data_pipeline = monomer_data_pipeline
@@ -513,18 +674,16 @@ def main(argv):
   model_names = config.MODEL_PRESETS[FLAGS.model_preset]
   for model_name in model_names:
     model_config = config.model_config(model_name)
-    if run_multimer_system:
-      model_config.model.num_ensemble_eval = num_ensemble
-    else:
-      model_config.data.eval.num_ensemble = num_ensemble
     model_params = data.get_model_haiku_params(
-        model_name=model_name, data_dir=FLAGS.data_dir)
+        model_name=model_name, data_dir=FLAGS.data_dir
+    )
     model_runner = model.RunModel(model_config, model_params)
     for i in range(num_predictions_per_model):
       model_runners[f'{model_name}_pred_{i}'] = model_runner
 
-  logging.info('Have %d models: %s', len(model_runners),
-               list(model_runners.keys()))
+  logging.info(
+      'Have %d models: %s', len(model_runners), list(model_runners.keys())
+  )
 
   amber_relaxer = relax.AmberRelaxation(
       max_iterations=RELAX_MAX_ITERATIONS,
@@ -532,7 +691,8 @@ def main(argv):
       stiffness=RELAX_STIFFNESS,
       exclude_residues=RELAX_EXCLUDE_RESIDUES,
       max_outer_iterations=RELAX_MAX_OUTER_ITERATIONS,
-      use_gpu=FLAGS.use_gpu_relax)
+      use_gpu=FLAGS.use_gpu_relax,
+  )
 
   random_seed = FLAGS.random_seed
   if random_seed is None:
