@@ -127,11 +127,11 @@ class AlphaFoldBackend(FoldingBackend):
         model_name: str,
         num_cycle: int,
         model_dir: str,
-        num_multimer_predictions_per_model: int,
+        num_predictions_per_model: int,
         msa_depth_scan=False,
         model_names_custom: List[str] = None,
         msa_depth=None,
-        allow_resume: bool = True,
+        dropout=False,
         **kwargs,
     ) -> Dict:
         """
@@ -145,7 +145,7 @@ class AlphaFoldBackend(FoldingBackend):
             The number of recycling iterations to be used in prediction.
         model_dir : str
             The directory containing model parameters.
-        num_multimer_predictions_per_model : int
+        num_predictions_per_model : int
             The number of multimer predictions to perform for each model.
         msa_depth_scan : bool, optional
             Whether to adjust MSA depth logarithmically, default is False.
@@ -155,6 +155,8 @@ class AlphaFoldBackend(FoldingBackend):
             A specific MSA depth to use, default is None.
         allow_resume : bool, optional
             If set to True, resumes prediction from partially completed runs, default is True.
+        dropout : bool, optional
+            If set to True, use dropout when inferring for more diverse predictions, default is False.
         **kwargs : dict
             Additional keyword arguments for model runner configuration.
 
@@ -201,6 +203,8 @@ class AlphaFoldBackend(FoldingBackend):
             model_config = config.model_config(model_name)
             model_config.model.num_ensemble_eval = num_ensemble
             model_config["model"].update({"num_recycle": num_cycle})
+            if dropout:
+                model_config.model.global_config.eval_dropout = True
 
             model_params = data.get_model_haiku_params(
                 model_name=model_name, data_dir=model_dir
@@ -216,7 +220,7 @@ class AlphaFoldBackend(FoldingBackend):
                     np.logspace(
                         np.log10(16),
                         np.log10(num_msa),
-                        num_multimer_predictions_per_model,
+                        num_predictions_per_model,
                     )
                 ).astype(int)
 
@@ -224,11 +228,11 @@ class AlphaFoldBackend(FoldingBackend):
                     np.logspace(
                         np.log10(32),
                         np.log10(num_extra_msa),
-                        num_multimer_predictions_per_model,
+                        num_predictions_per_model,
                     )
                 ).astype(int)
 
-            for i in range(num_multimer_predictions_per_model):
+            for i in range(num_predictions_per_model):
                 logging.debug(f"msa_depth is type : {type(msa_depth)} value: {msa_depth}")
                 logging.debug(f"msa_depth_scan is type: {type(msa_depth_scan)} value: {msa_depth_scan}")
                 if msa_depth or msa_depth_scan:
@@ -397,6 +401,12 @@ class AlphaFoldBackend(FoldingBackend):
                 prediction_result['plddt'][:,
                                            None], residue_constants.atom_type_num, axis=-1
             )
+            
+            # Fix chain indexing: asym_id should be 0-based for proper chain assignment
+            # AlphaFold assigns asym_id starting from 1, but PDB chain IDs expect 0-based indexing
+            if 'asym_id' in processed_feature_dict:
+                processed_feature_dict['asym_id'] = processed_feature_dict['asym_id'] - 1
+            
             unrelaxed_protein = protein.from_prediction(
                 features=processed_feature_dict,
                 result=prediction_result,
@@ -427,13 +437,15 @@ class AlphaFoldBackend(FoldingBackend):
 
     @staticmethod
     def predict(model_runners: Dict,
-                objects_to_model: List[Dict[Union[MultimericObject, MonomericObject, ChoppedObject], str]],
+                objects_to_model: list,  # List of dicts with 'object' and 'output_dir'
                 allow_resume: bool,
                 skip_templates: bool,
                 random_seed: int = 42,
                 **kwargs):
-        for m in objects_to_model:
-            object_to_model, output_dir = next(iter(m.items()))
+        for entry in objects_to_model:
+            object_to_model = entry['object']
+            output_dir = entry['output_dir']
+            
             prediction_results = AlphaFoldBackend.predict_individual_job(
                 model_runners=model_runners,
                 multimeric_object=object_to_model,
@@ -444,8 +456,9 @@ class AlphaFoldBackend(FoldingBackend):
                 **kwargs
             )
             
-            yield {object_to_model: {"prediction_results": prediction_results,
-                                     "output_dir": output_dir}}
+            yield {'object': object_to_model, 
+                   'prediction_results': prediction_results,
+                   'output_dir': output_dir}
             
     @staticmethod
     def recalculate_confidence(prediction_results: Dict, multimer_mode:bool, 
